#include <iostream>
#include <cstdlib>
#include <iomanip>
#include <vector>
#include <ctime>

#define rep(i,a,b) for(int i=a;i<b;++i)

/*
	The Robinson-Schensted-Knuth correspondence :
	{permutations} -> {pairs of standard Young tableaux}
		p |-> (P,Q) 
*/

using namespace std;

typedef vector<int> vi;

ostream& operator<<(ostream& os, const vector<vi >& T) {
	const int W = 4;
	rep(i,0,T.size()) {
		rep(j,0,T[i].size()*(W+1)+1)
			os << "-";
		os << endl;
		rep(j,0,T[i].size())
			os << "|" << setw(W) << right << (1+T[i][j]);
		os << "|" << endl; 
	}
	return os;
}

ostream& operator<<(ostream& os, const vi& v) {
	rep(i,0,v.size())
		os << (1+v[i]) << ' ';
	return os;
}

void rsk(vi& p) {//should return what's called P and Q at the end of the function scope (but pair<,> is so unwieldy that it doesn't)
	vector<vi> P, Q;
	rep(i,0,p.size()) {
		int x = p[i];
		int R = 0;
		int C;
		bool done = false;
		while(!done) {
			C = 0;
			if(R == P.size())
				P.push_back(vi(0)), Q.push_back(vi(0));
			while(C < P[R].size() && P[R][C] < x)
				++C;
			if(C == P[R].size())
				P[R].push_back(x), Q[R].push_back(i), done = true;
			else {
				int y = P[R][C];
				P[R][C] = x;
				x = y;
			}
			++R;
		}		
	}
	cout << "P: " << endl << P;
	cout << "Q: " << endl << Q;
}

vi rnd(int n) {//chooses an n-permutation uniformly at random
	vi rt(n);
	rep(i,0,n) {
		int j = rand()%(i+1);
		rt[i] = rt[j];
		rt[j] = i;
	}
	return rt;	
}

int main() {
	srand(time(0));
	vi p = rnd(12);
	cout << "p = " << p << endl;
	rsk(p);
	return 0;
}
